@@ -54,9 +54,7 @@ struct PowerManagerButtonPrivate
 #ifdef XFCE_PLUGIN
     XfcePanelPlugin *plugin;
 #endif
-#ifdef LXDE_PLUGIN
-    Plugin *plugin;
-#endif
+
     XfconfChannel   *channel;
 
     UpClient        *upower;
@@ -82,6 +80,14 @@ struct PowerManagerButtonPrivate
 
     /* display brightness slider widget */
     GtkWidget       *range;
+    /* Some laptops (and mostly newer ones with intel graphics) can turn off the
+     * backlight completely. If the user is not careful and sets the brightness
+     * very low using the slider, he might not be able to see the screen contents
+     * anymore. Brightness keys do not work on every laptop, so it's better to use
+     * a safe default minimum level that the user can change via the settings
+     * editor if desired.
+     */
+    gint32           brightness_min_level;
 
     /* filter range value changed events for snappier UI feedback */
     guint            set_level_timeout;
@@ -89,14 +95,23 @@ struct PowerManagerButtonPrivate
 
 typedef struct
 {
-    GdkPixbuf   *pix;          /* Icon */
-    gchar       *details;      /* Description of the device + state */
-    gchar       *object_path;  /* UpDevice object path */
-    UpDevice    *device;       /* Pointer to the UpDevice */
-    gulong       signal_id;    /* device changed callback id */
-    GtkWidget   *menu_item;    /* The device's item on the menu (if shown) */
+    GdkPixbuf   *pix;               /* Icon */
+    GtkWidget   *img;               /* Icon image in the menu */
+    gchar       *details;           /* Description of the device + state */
+    gchar       *object_path;       /* UpDevice object path */
+    UpDevice    *device;            /* Pointer to the UpDevice */
+    gulong       changed_signal_id; /* device changed callback id */
+    gulong       expose_signal_id;  /* expose-event callback id */
+    GtkWidget   *menu_item;         /* The device's item on the menu (if shown) */
 } BatteryDevice;
 
+typedef enum
+{
+    PROP_0 = 0,
+    PROP_BRIGHTNESS_MIN_LEVEL,
+} POWER_MANAGER_BUTTON_PROPERTIES;
+
+
 G_DEFINE_TYPE (PowerManagerButton, power_manager_button, GTK_TYPE_TOGGLE_BUTTON)
 
 static void power_manager_button_finalize   (GObject *object);
@@ -105,10 +120,10 @@ static gboolean power_manager_button_device_icon_expose (GtkWidget *img, GdkEven
 static gboolean power_manager_button_set_icon (PowerManagerButton *button);
 static gboolean power_manager_button_press_event (GtkWidget *widget, GdkEventButton *event);
 static void power_manager_button_show_menu (PowerManagerButton *button);
-static void power_manager_button_menu_add_device (PowerManagerButton *button, BatteryDevice *battery_device, gboolean append);
+static gboolean power_manager_button_menu_add_device (PowerManagerButton *button, BatteryDevice *battery_device, gboolean append);
 static void increase_brightness (PowerManagerButton *button);
 static void decrease_brightness (PowerManagerButton *button);
-
+static void battery_device_remove_pix (BatteryDevice *battery_device);
 
 
 static BatteryDevice*
@@ -118,35 +133,45 @@ get_display_device (PowerManagerButton *button)
     gdouble highest_percentage = 0;
     BatteryDevice *display_device = NULL;
 
+    TRACE("entering");
+
+    g_return_val_if_fail ( POWER_MANAGER_IS_BUTTON(button), NULL );
+
     if (button->priv->display_device)
     {
-	item = find_device_in_list (button, up_device_get_object_path (button->priv->display_device));
-	if (item)
-	    return item->data;
+        item = find_device_in_list (button, up_device_get_object_path (button->priv->display_device));
+        if (item)
+        {
+            return item->data;
+        }
     }
 
     /* We want to find the battery or ups device with the highest percentage
      * and use that to get our tooltip from */
     for (item = g_list_first (button->priv->devices); item != NULL; item = g_list_next (item))
     {
-	BatteryDevice *battery_device = item->data;
-	guint type = 0;
-	gdouble percentage;
+        BatteryDevice *battery_device = item->data;
+        guint type = 0;
+        gdouble percentage;
 
-	g_object_get (battery_device->device,
-		      "kind", &type,
-		      "percentage", &percentage,
-		      NULL);
+        if (!battery_device->device || !UP_IS_DEVICE(battery_device))
+        {
+            continue;
+        }
 
-	if ( type == UP_DEVICE_KIND_BATTERY || type == UP_DEVICE_KIND_UPS)
-	{
-	    if ( highest_percentage < percentage )
-	    {
-		/* found something better */
-		display_device = battery_device;
-		highest_percentage = percentage;
-	    }
-	}
+        g_object_get (battery_device->device,
+                      "kind", &type,
+                      "percentage", &percentage,
+                      NULL);
+
+        if ( type == UP_DEVICE_KIND_BATTERY || type == UP_DEVICE_KIND_UPS)
+        {
+            if ( highest_percentage < percentage )
+            {
+                display_device = battery_device;
+                highest_percentage = percentage;
+            }
+        }
     }
 
     return display_device;
@@ -157,16 +182,26 @@ power_manager_button_set_tooltip (PowerManagerButton *button)
 {
     BatteryDevice *display_device = get_display_device (button);
 
+    TRACE("entering");
+
+    if (!GTK_IS_WIDGET (button))
+    {
+        g_critical ("power_manager_button_set_tooltip: !GTK_IS_WIDGET (button)");
+        return;
+    }
+
     if ( display_device )
     {
-	/* if we have something, display it */
-	gtk_widget_set_tooltip_markup (GTK_WIDGET (button), display_device->details);
-    }
-    else
-    {
-	/* how did we get here? */
-	gtk_widget_set_tooltip_text (GTK_WIDGET (button), _("Display battery levels for attached devices"));
+        /* if we have something, display it */
+        if( display_device->details )
+        {
+            gtk_widget_set_tooltip_markup (GTK_WIDGET (button), display_device->details);
+            return;
+        }
     }
+
+    /* Odds are this is a desktop without any batteries attached */
+    gtk_widget_set_tooltip_text (GTK_WIDGET (button), _("Display battery levels for attached devices"));
 }
 
 static GList*
@@ -180,9 +215,15 @@ find_device_in_list (PowerManagerButton *button, const gchar *object_path)
 
     for (item = g_list_first (button->priv->devices); item != NULL; item = g_list_next (item))
     {
-	BatteryDevice *battery_device = item->data;
-	if (g_strcmp0 (battery_device->object_path, object_path) == 0)
-	    return item;
+        BatteryDevice *battery_device = item->data;
+        if (battery_device == NULL)
+        {
+            DBG("!battery_device");
+            continue;
+        }
+
+        if (g_strcmp0 (battery_device->object_path, object_path) == 0)
+            return item;
     }
 
     return NULL;
@@ -192,58 +233,100 @@ static gboolean
 power_manager_button_device_icon_expose (GtkWidget *img, GdkEventExpose *event, gpointer userdata)
 {
     cairo_t *cr;
-    UpDevice *device = UP_DEVICE(userdata);
-    guint type = 0;
+    UpDevice *device = NULL;
+    guint type = 0, state = 0;
     gdouble percentage;
     gint height, width;
     gdouble min_height = 2;
+    PangoLayout *layout = NULL;
+    PangoRectangle ink_extent, log_extent;
 
-    TRACE("entering for %s", up_device_get_object_path(device));
-    g_object_get (device,
-                  "kind", &type,
-                  "percentage", &percentage,
-		  NULL);
+    TRACE("entering");
 
-    /* Don't draw the progressbar for Battery and UPS */
-    if (type == UP_DEVICE_KIND_BATTERY || type == UP_DEVICE_KIND_UPS)
+    /* sanity checks */
+    if (!img || !GTK_IS_WIDGET (img))
         return FALSE;
 
+    if (UP_IS_DEVICE (userdata))
+    {
+        device = UP_DEVICE(userdata);
+
+        g_object_get (device,
+                      "kind", &type,
+                      "state", &state,
+                      "percentage", &percentage,
+                      NULL);
+
+        /* Don't draw the progressbar for Battery and UPS */
+        if (type == UP_DEVICE_KIND_BATTERY || type == UP_DEVICE_KIND_UPS)
+            return FALSE;
+    }
+    else
+    {
+        /* If the UpDevice hasn't fully updated yet it then we'll want
+         * a question mark for sure. */
+        state = UP_DEVICE_STATE_UNKNOWN;
+    }
+
     cr = gdk_cairo_create (img->window);
     width = img->allocation.width;
     height = img->allocation.height;
 
-    /* Draw the trough of the progressbar */
-    cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
-    cairo_set_line_width (cr, 1.0);
-    cairo_rectangle (cr, width - 3.5, img->allocation.y + 1.5, 5, height - 2);
-    cairo_set_source_rgb (cr, 0.87, 0.87, 0.87);
-    cairo_fill_preserve (cr);
-    cairo_set_source_rgb (cr, 0.53, 0.54, 0.52);
-    cairo_stroke (cr);
+    if (state != UP_DEVICE_STATE_UNKNOWN)
+    {
+        /* Draw the trough of the progressbar */
+        cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+        cairo_set_line_width (cr, 1.0);
+        cairo_rectangle (cr, width - 3.5, img->allocation.y + 1.5, 5, height - 2);
+        cairo_set_source_rgb (cr, 0.87, 0.87, 0.87);
+        cairo_fill_preserve (cr);
+        cairo_set_source_rgb (cr, 0.53, 0.54, 0.52);
+        cairo_stroke (cr);
 
-    /* Draw the fill of the progressbar
-       Use yellow for 20% and below, green for 100%, red for 5% and below and blue for the rest */
-    cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+        /* Draw the fill of the progressbar
+           Use yellow for 20% and below, green for 100%, red for 5% and below and blue for the rest */
+        cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
 
-    if ((height * (percentage / 100)) > min_height)
-       min_height = (height - 3) * (percentage / 100);
+        if ((height * (percentage / 100)) > min_height)
+           min_height = (height - 3) * (percentage / 100);
 
-    cairo_rectangle (cr, width - 3, img->allocation.y + height - min_height - 1, 4, min_height);
-    if (percentage > 5 && percentage < 20)
-        cairo_set_source_rgb (cr, 0.93, 0.83, 0.0);
-    else if (percentage > 20 && percentage < 100)
-        cairo_set_source_rgb (cr, 0.2, 0.4, 0.64);
-    else if (percentage == 100)
-        cairo_set_source_rgb (cr, 0.45, 0.82, 0.08);
+        cairo_rectangle (cr, width - 3, img->allocation.y + height - min_height - 1, 4, min_height);
+        if (percentage > 5 && percentage < 20)
+            cairo_set_source_rgb (cr, 0.93, 0.83, 0.0);
+        else if (percentage > 20 && percentage < 100)
+            cairo_set_source_rgb (cr, 0.2, 0.4, 0.64);
+        else if (percentage == 100)
+            cairo_set_source_rgb (cr, 0.45, 0.82, 0.08);
+        else
+            cairo_set_source_rgb (cr, 0.94, 0.16, 0.16);
+        cairo_fill (cr);
+
+        cairo_rectangle (cr, width - 2.5, img->allocation.y + 2.5, 3, height - 4);
+        cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.75);
+        cairo_stroke (cr);
+    }
     else
-        cairo_set_source_rgb (cr, 0.94, 0.16, 0.16);
-    cairo_fill (cr);
+    {
+        /* Draw a bubble with a question mark for devices with unknown state */
+        cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+        cairo_set_line_width (cr, 1.0);
+        cairo_arc(cr, width - 4.5, img->allocation.y + 6.5, 6, 0, 2*3.14159);
+        cairo_set_source_rgb (cr, 0.2, 0.54, 0.9);
+        cairo_fill_preserve (cr);
+        cairo_set_source_rgb (cr, 0.1, 0.37, 0.6);
+        cairo_stroke (cr);
 
-    cairo_rectangle (cr, width - 2.5, img->allocation.y + 2.5, 3, height - 4);
-    cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.75);
-    cairo_stroke (cr);
+        layout = gtk_widget_create_pango_layout (GTK_WIDGET (img), "?");
+        pango_layout_set_font_description (layout, pango_font_description_from_string ("Sans Bold 9"));
+        pango_layout_get_pixel_extents (layout, &ink_extent, &log_extent);
+        cairo_move_to (cr, (width - 5.5) - (log_extent.width / 2), (img->allocation.y + 5.5) - (log_extent.height / 2));
+        cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
+        pango_cairo_show_layout (cr, layout);
+    }
 
     cairo_destroy (cr);
+    if (layout)
+        g_object_unref (layout);
     return FALSE;
 }
 
@@ -287,8 +370,9 @@ power_manager_button_update_device_icon_and_details (PowerManagerButton *button,
 	g_free(battery_device->details);
     battery_device->details = details;
 
-    if (battery_device->pix)
-	g_object_unref (battery_device->pix);
+    /* If we had an image before, remove it and the callback */
+    battery_device_remove_pix(battery_device);
+
     battery_device->pix = pix;
 
     /* Get the display device, which may now be this one */
@@ -307,14 +391,17 @@ power_manager_button_update_device_icon_and_details (PowerManagerButton *button,
     /* If the menu is being displayed, update it */
     if (button->priv->menu && battery_device->menu_item)
     {
-	GtkWidget *img;
+        gtk_menu_item_set_label (GTK_MENU_ITEM (battery_device->menu_item), details);
 
-	gtk_menu_item_set_label (GTK_MENU_ITEM (battery_device->menu_item), details);
-
-        /* update the image */
-        img = gtk_image_new_from_pixbuf(battery_device->pix);
-        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(battery_device->menu_item), img);
-        g_signal_connect_after (G_OBJECT (img), "expose-event", G_CALLBACK (power_manager_button_device_icon_expose), device);
+        /* update the image, keep track of the signal ids and the img
+         * so we can disconnect it later */
+        battery_device->img = gtk_image_new_from_pixbuf(battery_device->pix);
+        g_object_ref (battery_device->img);
+        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(battery_device->menu_item), battery_device->img);
+        battery_device->expose_signal_id = g_signal_connect_after (G_OBJECT (battery_device->img),
+                                                                   "expose-event",
+                                                                   G_CALLBACK (power_manager_button_device_icon_expose),
+                                                                   device);
     }
 }
 
@@ -360,8 +447,8 @@ power_manager_button_add_device (UpDevice *device, PowerManagerButton *button)
 
     /* populate the struct */
     battery_device->object_path = g_strdup (object_path);
-    battery_device->signal_id = signal_id;
-    battery_device->device = device;
+    battery_device->changed_signal_id = signal_id;
+    battery_device->device = g_object_ref(device);
 
     /* add it to the list */
     button->priv->devices = g_list_append (button->priv->devices, battery_device);
@@ -376,6 +463,33 @@ power_manager_button_add_device (UpDevice *device, PowerManagerButton *button)
     }
 }
 
+/* This function unrefs the pix and img from the battery device and
+ * disconnects the expose-event callback on the img.
+ */
+static void
+battery_device_remove_pix (BatteryDevice *battery_device)
+{
+    TRACE("entering");
+
+    if (battery_device == NULL)
+        return;
+
+    if (G_IS_OBJECT(battery_device->pix))
+    {
+        if (GTK_IS_WIDGET(battery_device->img))
+        {
+            if (battery_device->expose_signal_id != 0)
+            {
+                g_signal_handler_disconnect (battery_device->img, battery_device->expose_signal_id);
+                battery_device->expose_signal_id = 0;
+            }
+            g_object_unref (battery_device->img);
+            battery_device->img = NULL;
+        }
+        battery_device->pix = NULL;
+    }
+}
+
 static void
 power_manager_button_remove_device (PowerManagerButton *button, const gchar *object_path)
 {
@@ -395,17 +509,18 @@ power_manager_button_remove_device (PowerManagerButton *button, const gchar *obj
     if(battery_device->menu_item && button->priv->menu)
         gtk_container_remove(GTK_CONTAINER(button->priv->menu), battery_device->menu_item);
 
-    if (battery_device->pix)
-	g_object_unref (battery_device->pix);
-
     g_free(battery_device->details);
     g_free(battery_device->object_path);
 
-    if (battery_device->device)
+    if (battery_device->device != NULL && UP_IS_DEVICE(battery_device->device))
     {
-	if (battery_device->signal_id)
-	    g_signal_handler_disconnect (battery_device->device, battery_device->signal_id);
-	g_object_unref (battery_device->device);
+        /* disconnect the signal handler if we were using it */
+        if (battery_device->changed_signal_id != 0)
+            g_signal_handler_disconnect (battery_device->device, battery_device->changed_signal_id);
+        battery_device->changed_signal_id = 0;
+
+        g_object_unref (battery_device->device);
+        battery_device->device = NULL;
     }
 
     /* remove it item and free the battery device */
@@ -482,7 +597,7 @@ brightness_down (PowerManagerButton *button)
     gint32 level;
     xfpm_brightness_get_level (button->priv->brightness, &level);
 
-    if ( level > 0 )
+    if ( level > button->priv->brightness_min_level )
     {
         decrease_brightness (button);
     }
@@ -514,6 +629,76 @@ power_manager_button_scroll_event (GtkWidget *widget, GdkEventScroll *ev)
     return FALSE;
 }
 
+static void
+set_brightness_min_level(PowerManagerButton *button, gint32 new_brightness_level)
+{
+    gint32 max_level = xfpm_brightness_get_max_level (button->priv->brightness);
+
+    /* sanity check */
+    if (new_brightness_level > max_level)
+        new_brightness_level = -1;
+
+    /* -1 = auto, we set the step value to a hopefully sane default */
+    if (new_brightness_level == -1)
+    {
+        button->priv->brightness_min_level = (max_level > 100) ? SAFE_SLIDER_MIN_LEVEL : 0;
+    }
+    else
+    {
+        button->priv->brightness_min_level = new_brightness_level;
+    }
+
+    DBG("button->priv->brightness_min_level : %d", button->priv->brightness_min_level);
+
+    /* update the range if it's being shown */
+    if (button->priv->range)
+    {
+        gtk_range_set_range (GTK_RANGE(button->priv->range), button->priv->brightness_min_level, max_level);
+    }
+}
+
+static void
+power_manager_button_set_property(GObject *object,
+                                  guint property_id,
+                                  const GValue *value,
+                                  GParamSpec *pspec)
+{
+    PowerManagerButton *button;
+
+    button = POWER_MANAGER_BUTTON (object);
+
+    switch(property_id)
+    {
+        case PROP_BRIGHTNESS_MIN_LEVEL:
+            set_brightness_min_level (button, g_value_get_int(value));
+            break;
+        default:
+            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+            break;
+    }
+}
+
+static void
+power_manager_button_get_property(GObject *object,
+                                  guint property_id,
+                                  GValue *value,
+                                  GParamSpec *pspec)
+{
+    PowerManagerButton *button;
+
+    button = POWER_MANAGER_BUTTON (object);
+
+    switch(property_id)
+    {
+        case PROP_BRIGHTNESS_MIN_LEVEL:
+            g_value_set_int(value, button->priv->brightness_min_level);
+            break;
+        default:
+            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+            break;
+    }
+}
+
 static void
 power_manager_button_class_init (PowerManagerButtonClass *klass)
 {
@@ -521,11 +706,29 @@ power_manager_button_class_init (PowerManagerButtonClass *klass)
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
     object_class->finalize = power_manager_button_finalize;
+    object_class->set_property = power_manager_button_set_property;
+    object_class->get_property = power_manager_button_get_property;
 
     widget_class->button_press_event = power_manager_button_press_event;
     widget_class->scroll_event = power_manager_button_scroll_event;
 
     g_type_class_add_private (klass, sizeof (PowerManagerButtonPrivate));
+
+#define XFPM_PARAM_FLAGS  (G_PARAM_READWRITE \
+                           | G_PARAM_CONSTRUCT \
+                           | G_PARAM_STATIC_NAME \
+                           | G_PARAM_STATIC_NICK \
+                           | G_PARAM_STATIC_BLURB)
+
+    /* We allow and default to -1 only so that we can automagically set a
+     * sane value if the user hasn't selected one already */
+    g_object_class_install_property(object_class, PROP_BRIGHTNESS_MIN_LEVEL,
+                                    g_param_spec_int(BRIGHTNESS_SLIDER_MIN_LEVEL,
+                                                     BRIGHTNESS_SLIDER_MIN_LEVEL,
+                                                     BRIGHTNESS_SLIDER_MIN_LEVEL,
+                                                     -1, G_MAXINT32, -1,
+                                                     XFPM_PARAM_FLAGS));
+#undef XFPM_PARAM_FLAGS
 }
 
 static void
@@ -592,7 +795,7 @@ GtkWidget *
 power_manager_button_new (XfcePanelPlugin *plugin)
 #endif
 #ifdef LXDE_PLUGIN
-power_manager_button_new (Plugin *plugin)
+power_manager_button_new (void)
 #endif
 {
     PowerManagerButton *button = NULL;
@@ -601,9 +804,10 @@ power_manager_button_new (Plugin *plugin)
 #ifdef XFCE_PLUGIN
     button->priv->plugin = XFCE_PANEL_PLUGIN (g_object_ref (plugin));
 #endif
-#ifdef LXDE_PLUGIN
-    button->priv->plugin = plugin;
-#endif
+
+    xfconf_g_property_bind(button->priv->channel,
+                           PROPERTIES_PREFIX BRIGHTNESS_SLIDER_MIN_LEVEL, G_TYPE_INT,
+                           G_OBJECT(button), BRIGHTNESS_SLIDER_MIN_LEVEL);
 
     return GTK_WIDGET (button);
 }
@@ -690,7 +894,11 @@ power_manager_button_free_data_cb (XfcePanelPlugin *plugin, PowerManagerButton *
 static void
 help_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
+#if LIBXFCE4UI_CHECK_VERSION(4, 11, 1)
+    xfce_dialog_show_help_with_version (NULL, "xfce4-power-manager", "start", NULL, XFPM_VERSION_SHORT);
+#else
     xfce_dialog_show_help (NULL, "xfce4-power-manager", "start", NULL);
+#endif
 }
 
 void
@@ -787,22 +995,30 @@ menu_item_activate_cb(GtkWidget *object, gpointer user_data)
     }
 }
 
-static void
+static gboolean
 power_manager_button_menu_add_device (PowerManagerButton *button, BatteryDevice *battery_device, gboolean append)
 {
-    GtkWidget *mi, *label, *img;
+    GtkWidget *mi, *label;
     guint type = 0;
 
+    g_return_val_if_fail (POWER_MANAGER_IS_BUTTON (button), FALSE);
+
     /* We need a menu to attach it to */
-    g_return_if_fail (button->priv->menu);
+    g_return_val_if_fail (button->priv->menu, FALSE);
 
-    g_object_get (battery_device->device,
-		  "kind", &type,
-		  NULL);
+    if (UP_IS_DEVICE(battery_device->device))
+    {
+        g_object_get (battery_device->device,
+                      "kind", &type,
+                      NULL);
 
-    /* Don't add the display device or line power to the menu */
-    if (type == UP_DEVICE_KIND_LINE_POWER || battery_device->device == button->priv->display_device)
-	return;
+        /* Don't add the display device or line power to the menu */
+        if (type == UP_DEVICE_KIND_LINE_POWER || battery_device->device == button->priv->display_device)
+        {
+            DBG("filtering device from menu (display or line power device)");
+            return FALSE;
+        }
+    }
 
     mi = gtk_image_menu_item_new_with_label(battery_device->details);
     /* Make the menu item be bold and multi-line */
@@ -810,13 +1026,17 @@ power_manager_button_menu_add_device (PowerManagerButton *button, BatteryDevice
     gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
 
     /* add the image */
-    img = gtk_image_new_from_pixbuf(battery_device->pix);
-    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+    battery_device->img = gtk_image_new_from_pixbuf(battery_device->pix);
+    g_object_ref (battery_device->img);
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), battery_device->img);
 
     /* keep track of the menu item in the battery_device so we can update it */
     battery_device->menu_item = mi;
     g_signal_connect(G_OBJECT(mi), "destroy", G_CALLBACK(menu_item_destroyed_cb), button);
-    g_signal_connect_after (G_OBJECT (img), "expose-event", G_CALLBACK (power_manager_button_device_icon_expose), battery_device->device);
+    battery_device->expose_signal_id = g_signal_connect_after (G_OBJECT (battery_device->img),
+                                                               "expose-event",
+                                                               G_CALLBACK (power_manager_button_device_icon_expose),
+                                                               battery_device->device);
 
     /* Active calls xfpm settings with the device's id to display details */
     g_signal_connect(G_OBJECT(mi), "activate", G_CALLBACK(menu_item_activate_cb), button);
@@ -827,6 +1047,8 @@ power_manager_button_menu_add_device (PowerManagerButton *button, BatteryDevice
 	gtk_menu_shell_append(GTK_MENU_SHELL(button->priv->menu), mi);
     else
 	gtk_menu_shell_prepend(GTK_MENU_SHELL(button->priv->menu), mi);
+
+    return TRUE;
 }
 
 static void
@@ -841,7 +1063,7 @@ decrease_brightness (PowerManagerButton *button)
 
     xfpm_brightness_get_level (button->priv->brightness, &level);
 
-    if ( level > 0 )
+    if ( level > button->priv->brightness_min_level )
     {
         xfpm_brightness_down (button->priv->brightness, &level);
         if (button->priv->range)
@@ -941,7 +1163,11 @@ power_manager_button_show_menu (PowerManagerButton *button)
     GdkScreen *gscreen;
     GList *item;
     gboolean show_separator_flag = FALSE;
-    gint32 min_level, max_level, current_level = 0;
+    gint32 max_level, current_level = 0;
+
+    TRACE("entering");
+
+    g_return_if_fail (POWER_MANAGER_IS_BUTTON (button));
 
     if(gtk_widget_has_screen(GTK_WIDGET(button)))
         gscreen = gtk_widget_get_screen(GTK_WIDGET(button));
@@ -958,9 +1184,11 @@ power_manager_button_show_menu (PowerManagerButton *button)
     {
         BatteryDevice *battery_device = item->data;
 
-        power_manager_button_menu_add_device (button, battery_device, TRUE);
-        /* If we add an item to the menu, show the separator */
-        show_separator_flag = TRUE;
+        if (power_manager_button_menu_add_device (button, battery_device, TRUE))
+        {
+            /* If we add an item to the menu, show the separator */
+            show_separator_flag = TRUE;
+        }
     }
 
     if (show_separator_flag)
@@ -978,26 +1206,7 @@ power_manager_button_show_menu (PowerManagerButton *button)
 
         max_level = xfpm_brightness_get_max_level (button->priv->brightness);
 
-        /* determine minimum value for slider */
-        min_level = xfconf_channel_get_int (button->priv->channel,
-                                            PROPERTIES_PREFIX BRIGHTNESS_SLIDER_MIN_LEVEL,
-                                            -1);
-        if (min_level == -1)
-        {
-            /* Some laptops (and mostly newer ones with intel graphics) can turn off the
-             * backlight completely. If the user is not careful and sets the brightness
-             * very low using the slider, he might not be able to see the screen contents
-             * anymore. Brightness keys do not work on every laptop, so it's better to use
-             * a safe default minimum level that the user can change via the settings
-             * editor if desired.
-             */
-            min_level = (max_level > 100) ? SAFE_SLIDER_MIN_LEVEL : 0;
-            xfconf_channel_set_int (button->priv->channel,
-                                    PROPERTIES_PREFIX BRIGHTNESS_SLIDER_MIN_LEVEL,
-                                    min_level);
-        }
-
-        mi = scale_menu_item_new_with_range (min_level, max_level, 1);
+        mi = scale_menu_item_new_with_range (button->priv->brightness_min_level, max_level, 1);
 
         /* attempt to load and display the brightness icon */
         pix = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
@@ -1051,7 +1260,11 @@ power_manager_button_show_menu (PowerManagerButton *button)
 #else
                     NULL,
 #endif
+#ifdef XFCE_PLUGIN
                     button->priv->plugin,
+#else
+                    NULL,
+#endif
                     0,
                     gtk_get_current_event_time ());
 }
